/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.gui;

import bardp.gui.FrameGui;
import bardp.gui.FrameGui.ExecutionModel;
import bardp.img.coor.CoorRange;
import bardp.img.controller.FrameController;
import bardp.protocol.ProtocolFrame;
import bardp.protocol.ProtocolList;
import bardp.protocol.protocolTexture.TextureController;
import bardp.protocol.protocolTexture.TextureControllerSub;
import bardp.protocol.protocolTexture.params.TextureJFrameOutputParameters;
import bardp.raw.RawTZYX;
import bardp.ressources.Res;
import bardp.roi.controller.RoiController;
import bardp.roi.tools.split3D.Split3DMainSub;
import bardp.session.patient.study.series.SeriesModel;
import bardp.session.patient.study.series.image.SpModel;
import bardp.utility.LIFExFonts;
import bardp.utility.MessageGui;
import bardp.utility.SvgAbstractButton;
import bardp.utility.Utils;
import bardp.utility.WorkDir;
import bardp.utility.settings.Settings;
import bardp.utility.settings.SettingsOption;
import bardp.utility.swing.WrapFlowLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureGlobalROIJPanel extends JPanel {

    /**
     * Creates new form TexJPanelOption
     */
    public TextureGlobalROIJPanel() {

        this.prefsOption = new SettingsOption();

        initComponents();
        postComponents();

        texJFrameOutputParameters = new TextureJFrameOutputParameters();
        jTextFieldZpatialResamplingCheck();
        jTextFieldYpatialResamplingCheck();
        jTextFieldXSpatialResamplingCheck();
    }

    public void setSeries(
            final SeriesModel series
    ) {
        this.series = series;

        //<editor-fold defaultstate="collapsed" desc="enable setting">
        this.jButtonIntensityDiscretizationDefaultSetting.setEnabled(false);
        this.jButtonIntensityResamplingDefaultSetting.setEnabled(false);

        boolean isPT_NM_CT
                = series.modality.equals("PT")
                || series.modality.equals("MN")
                || series.modality.equals("CT");
        this.jButtonIntensityDiscretizationDefaultSetting.setEnabled(isPT_NM_CT);
        this.jButtonIntensityResamplingDefaultSetting.setEnabled(isPT_NM_CT);

        boolean have3D = series.rawTZYX!=null && series.rawTZYX.nz > 1;
        this.jTextFieldZSpatialResampling.setEnabled(have3D);
        //</editor-fold> 

        filenameProperties = WorkDir.getRawVolumeSerieDirectory(series)
                + "Texture_Props.txt";
        setPropertiesValues();
    }

    private void postComponents() {

        this.setBackground(ProtocolList.TEXTURE.backgroundColorGui);
        this.setOpaque(true);
        
        this.setLayout(new WrapFlowLayout());
        this.jPanelInput.setLayout(new WrapFlowLayout());

        Border bGreyLine = BorderFactory.createLineBorder(Color.DARK_GRAY, 2, true);

        //<editor-fold defaultstate="collapsed" desc="SpatialResampling">
        SvgAbstractButton.setIcon(jButtonSpatialResamplingDefaultSetting, "Default setting", SvgAbstractButton.TEXT_HIDE,
                "Set default settings", // toolTipText
                "SettingsDefault", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM
        );
        SvgAbstractButton.setIcon(jButtonSpatialResamplingHelp, "Help", SvgAbstractButton.TEXT_HIDE,
                "<html>As most textural feature values depend on the voxel size,<br>"
                + "please select the voxel size to be used for calculation<br>"
                + "if you want to compare your values with values published by others.</html>", // toolTipText
                "Help", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM
        );

        this.jPanelSpatialResampling.setBorder(
                BorderFactory.createTitledBorder(bGreyLine, "Spatial Resampling",
                        TitledBorder.DEFAULT_JUSTIFICATION,
                        TitledBorder.DEFAULT_POSITION,
                        LIFExFonts.FONT_MEDIUM,
                        Color.DARK_GRAY)
        );

        this.jLabelZSpatialResampling.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelYSpatialResampling.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelXSpatialResampling.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldZSpatialResampling.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldYSpatialResampling.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldXSpatialResampling.setFont(LIFExFonts.FONT_SMALL);
        this.jButtonSpatialResamplingDefaultSetting.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="IntensityResampling">
        this.jButtonIntensityResamplingDefaultSetting.setFont(LIFExFonts.FONT_SMALL);
        SvgAbstractButton.setIcon(jButtonIntensityResamplingDefaultSetting, "Default setting", SvgAbstractButton.TEXT_HIDE,
                "Set default settings", // toolTipText
                "SettingsDefault", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM
        );

        SvgAbstractButton.setIcon(jButtonIntensityResamplingHelp, "Help", SvgAbstractButton.TEXT_HIDE,
                "<html>In PET, absolute resampling is preferred,<br>between 0 and the max SUV you expect in the VOI in your whole cohort.</html>", // toolTipText
                "Help", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM
        );

        this.jTextFieldNbGrey.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldBinSize.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldMinBound.setFont(LIFExFonts.FONT_SMALL);
        this.jTextFieldMaxBound.setFont(LIFExFonts.FONT_SMALL);
        this.jRadioButtonAbsoluteMinMax.setFont(LIFExFonts.FONT_SMALL);
        this.jRadioButtonRelativeMinMax.setFont(LIFExFonts.FONT_SMALL);
        this.jRadioButtonRelativeSd.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="IntensityDiscretization">
        this.jPanelIntensityDiscretization.setBorder(BorderFactory.createTitledBorder(bGreyLine, "Intensity Discretization",
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION,
                LIFExFonts.FONT_MEDIUM,
                Color.DARK_GRAY)
        );
        SvgAbstractButton.setIcon(jButtonIntensityDiscretizationDefaultSetting, "Default setting", SvgAbstractButton.TEXT_HIDE,
                "Set default settings", // toolTipText
                "SettingsDefault", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM
        );
        this.jButtonIntensityDiscretizationDefaultSetting.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelBinSize.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelNbGrey.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>        

        //<editor-fold defaultstate="collapsed" desc="IntensityRescaling">
        this.jPanelIntensityRescaling.setBorder(
                BorderFactory.createTitledBorder(bGreyLine, "Intensity Rescaling",
                        javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                        javax.swing.border.TitledBorder.DEFAULT_POSITION,
                        LIFExFonts.FONT_MEDIUM,
                        Color.DARK_GRAY)
        );
        SvgAbstractButton.setIcon(
                jButtonIntensityDiscretizationHelp, "Help", SvgAbstractButton.TEXT_HIDE,
                "In PET, 64 levels are usually appropriate.", // toolTipText
                "Help", FrameGui.svg.ICON_MINI_MINI_SIZE,
                SwingConstants.BOTTOM
        );
        this.jLabelMinBound.setFont(LIFExFonts.FONT_SMALL);
        this.jLabelMaxBound.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>

      
        
        //<editor-fold defaultstate="collapsed" desc="Dimension processing">
        this.jPanelDimensionProcessing.setBorder(
                BorderFactory.createTitledBorder(bGreyLine, "Dimension processing",
                        TitledBorder.DEFAULT_JUSTIFICATION,
                        TitledBorder.DEFAULT_POSITION,
                        LIFExFonts.FONT_MEDIUM,
                        Color.DARK_GRAY)
        );
        this.jRadioButtonDimensionProcessing2D.setFont(LIFExFonts.FONT_SMALL);
        this.jRadioButtonDimensionProcessing3D.setFont(LIFExFonts.FONT_SMALL);
        //</editor-fold>
        
        //<editor-fold defaultstate="collapsed" desc="Command">
        this.jPanelCommand.setBackground(ProtocolList.TEXTURE.backgroundColorGui.darker());
        this.jPanelCommand.setOpaque(true);
        SvgAbstractButton.setIcon(
                jToggleButtonOutputParameters, "Settings", SvgAbstractButton.TEXT_VISIBLE,
                "Adv parameters", // toolTipText
                "Settings", FrameGui.svg.ICON_MEDIUM_SIZE,
                SwingConstants.BOTTOM
        );
        
        SvgAbstractButton.setIcon(
                jButtonMenuItemCheckRoiForTexture,
                "CheckTex", SvgAbstractButton.TEXT_VISIBLE,
                "Check if parameters of activated ROI for texture processing are valid", // toolTipText
                "ROICheckForTexture", FrameGui.svg.ICON_MEDIUM_SIZE,
                SwingConstants.BOTTOM
        );
        
        SvgAbstractButton.setIcon(
                jButtonRun, "Run", SvgAbstractButton.TEXT_VISIBLE,
                "Run texture on ROI", // toolTipText
                "Run", FrameGui.svg.ICON_MEDIUM_SIZE,
                SwingConstants.BOTTOM
        );
        //</editor-fold>
        
        //<editor-fold defaultstate="collapsed" desc="same sizeX for all optionbox">
        int xSize = jPanelSpatialResampling.getPreferredSize().width;
        xSize = Math.max(xSize, this.jPanelIntensityRescaling.getPreferredSize().width);
        xSize = Math.max(xSize, jPanelIntensityDiscretization.getPreferredSize().width);
        xSize = Math.max(xSize, jPanelCommand.getPreferredSize().width);

        this.jPanelSpatialResampling.setPreferredSize(
                new Dimension(xSize, jPanelSpatialResampling.getPreferredSize().height)
        );
        this.jPanelIntensityDiscretization.setPreferredSize(
                new Dimension(xSize, jPanelIntensityDiscretization.getPreferredSize().height)
        );
        this.jPanelIntensityRescaling.setPreferredSize(
                new Dimension(xSize, jPanelIntensityRescaling.getPreferredSize().height)
        );
        this.jPanelDimensionProcessing.setPreferredSize(
                new Dimension(xSize, jPanelDimensionProcessing.getPreferredSize().height)
        );
        this.jPanelCommand.setPreferredSize(
                new Dimension(xSize, jPanelCommand.getPreferredSize().height)
        );
        //</editor-fold>
    }

    /**
     * set fields with all default (=validated) values function of modalities
     */
    private void setIntensityResamplingDefaultValues() {

        switch (series.modality) {
            case "PT":
            case "MN":
                setIntensityResamplingDefaultValuesPT_MN();
                break;
            case "CT":
                setIntensityResamplingDefaultValuesCT();
                break;
            default:
                setIntensityResamplingDefaultValuesOther();
                break;
        }
    }

    /**
     * set Properties Loaded Values From Scripting
     * <br>for new script (multiples series)
     *
     * @param properties
     * @param sessionImgKey
     */
    public void setPropertiesLoadedValuesFromScripting(
            final Properties properties,
            final String sessionImgKey
    ) {
        scriptingCsv = properties.getProperty("LIFEx.texture.SessionCsv");

        nbGreyLevels = Float.parseFloat(properties.getProperty("LIFEx.texture.NbGrey"));
        this.jTextFieldNbGrey.setText(String.valueOf(nbGreyLevels));

        binSize = Float.parseFloat(properties.getProperty("LIFEx.texture.BinSize"));
        this.jTextFieldBinSize.setText(String.valueOf(binSize));

        buttonAbsolute = Boolean.parseBoolean(properties.getProperty("LIFEx.texture.ButtonAbsolute"));
        this.jRadioButtonAbsoluteMinMax.setSelected(buttonAbsolute);
        if (buttonAbsolute) {
            minBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MinBound"));
            this.jTextFieldMinBound.setText(String.valueOf(minBound));

            maxBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MaxBound"));
            this.jTextFieldMaxBound.setText(String.valueOf(maxBound));
        }

        try {
            int distanceWithNeighbours = Integer.parseInt(
                    properties.getProperty("LIFEx.texture.GLCM.DistanceWithNeighbours")
            );
            texJFrameOutputParameters.texJPanelOutputParameters.jComboBoxDistance.setSelectedIndex(distanceWithNeighbours - 1);
        } catch (NumberFormatException ignore) { // if not present then default value of gui is got
        }

        buttonRelativeMinMax = Boolean.parseBoolean(properties.getProperty("LIFEx.texture.ButtonRelativeMinMax"));
        this.jRadioButtonRelativeMinMax.setSelected(buttonRelativeMinMax);

        buttonRelativeMeanSd = Boolean.parseBoolean(properties.getProperty("LIFEx.texture.ButtonRelativeMeanSd"));
        this.jRadioButtonRelativeSd.setSelected(buttonRelativeMeanSd);

        try {
            zSpatialResampling = Float.parseFloat(properties.getProperty(sessionImgKey + ".ZSpatialResampling"));
            if (zSpatialResampling == 0) {
                zSpatialResampling = series.rawTZYX.iopIppSp.spModel.getZSpacing();
            }
        } catch (NullPointerException ex) { // if ZSpatialResampling not exist
            zSpatialResampling = series.rawTZYX.iopIppSp.spModel.getZSpacing();
        }

        try {
            ySpatialResampling = Float.parseFloat(properties.getProperty(sessionImgKey + ".YSpatialResampling"));
            if (ySpatialResampling == 0) {
                ySpatialResampling = series.rawTZYX.iopIppSp.spModel.getYSpacing();
            }
        } catch (NullPointerException ex) { // if YSpatialResampling not exist
            ySpatialResampling = series.rawTZYX.iopIppSp.spModel.getYSpacing();
        }

        try {
            xSpatialResampling = Float.parseFloat(properties.getProperty(sessionImgKey + ".XSpatialResampling"));
            if (xSpatialResampling == 0) {
                xSpatialResampling = series.rawTZYX.iopIppSp.spModel.getXSpacing();
            }
        } catch (NullPointerException ex) { // if XSpatialResampling not exist
            xSpatialResampling = series.rawTZYX.iopIppSp.spModel.getXSpacing();
        }

        dimensionProcessing = properties.getProperty("LIFEx.texture.DimensionProcessing", "3D").trim().toUpperCase();
        if (dimensionProcessing.equals("2D")) {
            this.jRadioButtonDimensionProcessing2D.setSelected(true);
        } else {
            this.jRadioButtonDimensionProcessing3D.setSelected(true);
        }
    }

    /**
     * set Properties Loaded Values From Scripting
     * <br>keep to retro compatibility with old script (only one series)
     *
     * @param properties
     */
    public void setPropertiesLoadedValuesFromScripting(
            final Properties properties
    ) {
        scriptingCsv = properties.getProperty("LIFEx.texture.SessionCsv");

        nbGreyLevels = Float.parseFloat(properties.getProperty("LIFEx.texture.NbGrey"));
        this.jTextFieldNbGrey.setText(String.valueOf(nbGreyLevels));

        binSize = Float.parseFloat(properties.getProperty("LIFEx.texture.BinSize"));
        this.jTextFieldBinSize.setText(String.valueOf(binSize));

        buttonAbsolute = Boolean.parseBoolean(properties.getProperty("LIFEx.texture.ButtonAbsolute"));
        this.jRadioButtonAbsoluteMinMax.setSelected(buttonAbsolute);
        if (buttonAbsolute) {
            minBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MinBound"));
            this.jTextFieldMinBound.setText(String.valueOf(minBound));

            maxBound = Float.parseFloat(properties.getProperty("LIFEx.texture.MaxBound"));
            this.jTextFieldMaxBound.setText(String.valueOf(maxBound));
        }

        try {
            int distanceWithNeighbours = Integer.parseInt(
                    properties.getProperty("LIFEx.texture.GLCM.DistanceWithNeighbours"));
            texJFrameOutputParameters.texJPanelOutputParameters.jComboBoxDistance.setSelectedIndex(distanceWithNeighbours - 1);
        } catch (NumberFormatException ignore) { // if not present then default GUI value is got
        }

        buttonRelativeMinMax = Boolean.parseBoolean(properties.getProperty("LIFEx.texture.ButtonRelativeMinMax"));
        this.jRadioButtonRelativeMinMax.setSelected(buttonRelativeMinMax);

        buttonRelativeMeanSd = Boolean.parseBoolean(properties.getProperty("LIFEx.texture.ButtonRelativeMeanSd"));
        this.jRadioButtonRelativeSd.setSelected(buttonRelativeMeanSd);

        zSpatialResampling = Float.parseFloat(properties.getProperty("LIFEx.texture.ZSpatialResampling"));
        if (zSpatialResampling == 0) {
            zSpatialResampling = series.rawTZYX.iopIppSp.spModel.getZSpacing();
        }

        ySpatialResampling = Float.parseFloat(properties.getProperty("LIFEx.texture.YSpatialResampling"));
        if (ySpatialResampling == 0) {
            ySpatialResampling = series.rawTZYX.iopIppSp.spModel.getYSpacing();
        }

        xSpatialResampling = Float.parseFloat(properties.getProperty("LIFEx.texture.XSpatialResampling"));
        if (xSpatialResampling == 0) {
            xSpatialResampling = series.rawTZYX.iopIppSp.spModel.getXSpacing();
        }
        
        dimensionProcessing = properties.getProperty("LIFEx.texture.DimensionProcessing", "3D").trim().toUpperCase();
        if (dimensionProcessing.equals("2D")) {
            this.jRadioButtonDimensionProcessing2D.setSelected(true);
        } else {
            this.jRadioButtonDimensionProcessing3D.setSelected(true);
        }
    }

    private void setAllFieldValues() {
        SwingUtilities.invokeLater(() -> {

            this.jTextFieldNbGrey.setText(nbGreyLevels == NO_DEF ? NO_DEF_STRING : String.valueOf(nbGreyLevels));
            this.jTextFieldNbGrey.setCaretPosition(0);

            this.jTextFieldBinSize.setText(binSize == NO_DEF ? NO_DEF_STRING : String.valueOf(binSize));
            this.jTextFieldBinSize.setCaretPosition(0);

            this.jTextFieldMinBound.setText(minBound == NO_DEF ? NO_DEF_STRING : String.valueOf(minBound));
            this.jTextFieldMinBound.setCaretPosition(0);

            this.jTextFieldMaxBound.setText(maxBound == NO_DEF ? NO_DEF_STRING : String.valueOf(maxBound));
            this.jTextFieldMaxBound.setCaretPosition(0);

            this.jRadioButtonAbsoluteMinMax.setSelected(buttonAbsolute);
            this.jRadioButtonRelativeMinMax.setSelected(buttonRelativeMinMax);
            this.jRadioButtonRelativeSd.setSelected(buttonRelativeMeanSd);

            try {
                this.jTextFieldZSpatialResampling.setText(String.valueOf(zSpatialResampling));
                this.jTextFieldZSpatialResampling.setCaretPosition(0);
            } catch (NullPointerException ignore) {
            }

            try {
                this.jTextFieldYSpatialResampling.setText(String.valueOf(ySpatialResampling));
                this.jTextFieldYSpatialResampling.setCaretPosition(0);
            } catch (NullPointerException ignore) {
            }

            try {
                this.jTextFieldXSpatialResampling.setText(String.valueOf(xSpatialResampling));
                this.jTextFieldXSpatialResampling.setCaretPosition(0);
            } catch (NullPointerException ignore) {
            }

            if (this.jRadioButtonRelativeMinMax.isSelected()) {
                if (nbGreyLevels == 0) {
                    this.jTextFieldNbGrey.setText("---");
                }
                if (binSize == 0) {
                    this.jTextFieldBinSize.setText("---");
                }
            } else if (this.jRadioButtonRelativeSd.isSelected()) {
                if (nbGreyLevels == 0) {
                    this.jTextFieldNbGrey.setText("---");
                }
                if (binSize == 0) {
                    this.jTextFieldBinSize.setText("---");
                }
            }
        });

    }

    /**
     * set parameter values with properties values loaded from file
     */
    private void setPropertiesValues() {
        if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL) {
            setSpatialResamplingDefaultValues();
        }

        boolean isLoadingPropertiesSuccess = loadProperties();
        if (!isLoadingPropertiesSuccess) {
            setIntensityResamplingDefaultValues();
            setSpatialResamplingDefaultValues();
        }

        setAllFieldValues();
    }

    /**
     * set default values to PT && NM modality
     */
    private void setIntensityResamplingDefaultValuesPT_MN() {
        this.nbGreyLevels = 64f;
        this.binSize = 20f / this.nbGreyLevels;

        this.minBound = 0;
        this.maxBound = 20;

        this.buttonAbsolute = true;
        this.buttonRelativeMeanSd = false;
        this.buttonRelativeMinMax = false;
    }

    /**
     * set default values to CT modality
     */
    private void setIntensityResamplingDefaultValuesCT() {
        this.nbGreyLevels = 4000f / 10f;
        this.binSize = 10f;

        this.minBound = -1000;
        this.maxBound = 3000;

        this.buttonAbsolute = true;
        this.buttonRelativeMeanSd = false;
        this.buttonRelativeMinMax = false;
    }

    /**
     * set default values to the other modalities
     */
    private void setIntensityResamplingDefaultValuesOther() {
        this.nbGreyLevels = NO_DEF;
        this.binSize = NO_DEF;

        this.minBound = NO_DEF;
        this.maxBound = NO_DEF;

        this.buttonAbsolute = false;
        this.buttonRelativeMeanSd = false;
        this.buttonRelativeMinMax = true;
    }

    private void setSpatialResamplingDefaultValues() {
        //<editor-fold defaultstate="collapsed" desc="check is series!=null">
        if (series == null) {
            new MessageGui(
                    "Texture",
                    "No series is selected. Operation canceled",
                    MessageGui.WARNING_MESSAGE
            );
            return;
        }
        //</editor-fold>

        RawTZYX rawZYX = series.rawTZYX;
        //<editor-fold defaultstate="collapsed" desc="check is rawZYX!=null">
        if (rawZYX == null) {
            new MessageGui(
                    "Texture",
                    "No image in series. Operation canceled",
                    MessageGui.WARNING_MESSAGE
            );
            return;
        }
        //</editor-fold>

        zSpatialResampling = rawZYX.iopIppSp.spModel.getZSpacing();
        ySpatialResampling = rawZYX.iopIppSp.spModel.getYSpacing();
        xSpatialResampling = rawZYX.iopIppSp.spModel.getXSpacing();
    }

    private void jTextFieldXSpatialResamplingCheck() {
        jTextFieldXSpatialResampling.getDocument().addDocumentListener(new DocumentListener() {

            private double maxSpacing;
            private double minSpacing;
            private MessageGui messageGUI;

            @Override
            public void changedUpdate(DocumentEvent e) {
                // mandatory
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                check();
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                check();
            }

            private void check() {
                try {
                    if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL) {

                        //<editor-fold defaultstate="collapsed" desc="maxSpacing minSpacing">
                        maxSpacing = Math.max(series.rawTZYX.iopIppSp.spModel.getZSpacing(),
                                series.rawTZYX.iopIppSp.spModel.getYSpacing()
                        );
                        maxSpacing = Math.max(maxSpacing,
                                series.rawTZYX.iopIppSp.spModel.getXSpacing()
                        );

                        minSpacing = Math.min(series.rawTZYX.iopIppSp.spModel.getZSpacing(),
                                series.rawTZYX.iopIppSp.spModel.getYSpacing()
                        );
                        minSpacing = Math.min(minSpacing,
                                series.rawTZYX.iopIppSp.spModel.getXSpacing()
                        );
                        //</editor-fold>

                        //<editor-fold defaultstate="collapsed" desc="test range X">
                        try {
                            xSpatialResampling = Float.parseFloat(jTextFieldXSpatialResampling.getText());

                            if (xSpatialResampling > 0 && xSpatialResampling < 0.2f * minSpacing) {
                                messageError();
                            } else if (xSpatialResampling > 5f * maxSpacing) {
                                messageError();
                            } else {
                                messageOK();
                            }

                        } catch (NumberFormatException ex) {
                            xSpatialResampling = 0;
                        }
                        //</editor-fold>
                    }
                } catch (Exception ignore) {
                }
            }

            private void messageOK() {
                /*if (messageGUI != null) {
          messageGUI.end();
        }
        if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL) {
          messageGUI = new MessageGui(
                  "Texture message",
                  "X spatial resampling: " + xSpatialResampling + " mm is valid.",
                  MessageGui.OK_MESSAGE_QUICK
          );
        }*/
            }

            private void messageError() {
                if (messageGUI != null) {
                    messageGUI.end();
                }
                messageGUI = new MessageGui(
                        "Texture message",
                        "Range of X spatial resampling: " + xSpatialResampling + " mm is not valid.\n"
                        + "Min bound must be: "
                        // + " >= 0.2*min(spZ, spY, spX)"
                        + " >= " + Utils.d2s(0.2f * minSpacing, 2) + "\n"
                        + "Max bound must be: "
                        //+ " <= 5*max(spZ, spY, spX)"
                        + " <= " + Utils.d2s(5f * maxSpacing, 2),
                        MessageGui.WARNING_MESSAGE
                );
            }

        });
    }

    private void jTextFieldYpatialResamplingCheck() {
        jTextFieldYSpatialResampling.getDocument().addDocumentListener(new DocumentListener() {

            private double maxSpacing;
            private double minSpacing;
            private MessageGui messageGUI;

            @Override
            public void changedUpdate(DocumentEvent e) {
                // mandatory
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                check();
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                check();
            }

            private void check() {
                try {
                    if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL) {

                        //<editor-fold defaultstate="collapsed" desc="maxSpacing minSpacing">
                        maxSpacing = Math.max(series.rawTZYX.iopIppSp.spModel.getZSpacing(),
                                series.rawTZYX.iopIppSp.spModel.getYSpacing()
                        );
                        maxSpacing = Math.max(maxSpacing,
                                series.rawTZYX.iopIppSp.spModel.getXSpacing()
                        );

                        minSpacing = Math.min(series.rawTZYX.iopIppSp.spModel.getZSpacing(),
                                series.rawTZYX.iopIppSp.spModel.getYSpacing()
                        );
                        minSpacing = Math.min(minSpacing,
                                series.rawTZYX.iopIppSp.spModel.getXSpacing()
                        );
                        //</editor-fold>

                        //<editor-fold defaultstate="collapsed" desc="test range X">
                        try {
                            ySpatialResampling = Float.parseFloat(jTextFieldYSpatialResampling.getText());

                            if (ySpatialResampling > 0 && ySpatialResampling < 0.2f * minSpacing) {
                                messageError();
                            } else if (ySpatialResampling > 5f * maxSpacing) {
                                messageError();
                            } else {
                                messageOK();
                            }

                        } catch (NumberFormatException ex) {
                            ySpatialResampling = 0;
                        }
                        //</editor-fold>
                    }
                } catch (Exception ignore) {
                }
            }

            private void messageOK() {
                /* if (messageGUI != null) {
          messageGUI.end();
        }
        if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL) {
          messageGUI = new MessageGui(
                  "Texture message",
                  "Y spatial resampling: " + ySpatialResampling + " mm is valid.",
                  MessageGui.OK_MESSAGE_QUICK
          );
        }*/
            }

            private void messageError() {
                if (messageGUI != null) {
                    messageGUI.end();
                }
                messageGUI = new MessageGui(
                        "Texture message",
                        "Range of Y spatial resampling: " + ySpatialResampling + " mm is not valid.\n"
                        + "Min bound must be: "
                        //  + " >= 0.2*min(spZ, spY, spX)"
                        + " >= " + Utils.d2s(0.2f * minSpacing, 2) + "\n"
                        + "Max bound must be: "
                        //  + " <= 5*max(spZ, spY, spX)"
                        + " <= " + Utils.d2s(5f * maxSpacing, 2),
                        MessageGui.WARNING_MESSAGE
                );
            }

        });
    }

    private void jTextFieldZpatialResamplingCheck() {
        jTextFieldZSpatialResampling.getDocument().addDocumentListener(new DocumentListener() {

            private double maxSpacing;
            private double minSpacing;
            private MessageGui messageGUI;

            @Override
            public void changedUpdate(DocumentEvent e) {
                // mandatory
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                check();
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                check();
            }

            private void check() {
                try {
                    if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL) {

                        //<editor-fold defaultstate="collapsed" desc="maxSpacing minSpacing">
                        maxSpacing = Math.max(
                                series.rawTZYX.iopIppSp.spModel.getZSpacing(),
                                Math.max(series.rawTZYX.iopIppSp.spModel.getYSpacing(),
                                        series.rawTZYX.iopIppSp.spModel.getXSpacing())
                        );

                        minSpacing = Math.min(
                                series.rawTZYX.iopIppSp.spModel.getZSpacing(),
                                Math.min(series.rawTZYX.iopIppSp.spModel.getYSpacing(),
                                        series.rawTZYX.iopIppSp.spModel.getXSpacing())
                        );

                        //</editor-fold>
                        //<editor-fold defaultstate="collapsed" desc="test range X">
                        try {
                            zSpatialResampling = Float.parseFloat(jTextFieldZSpatialResampling.getText());

                            if (zSpatialResampling > 0 && zSpatialResampling < 0.2f * minSpacing) {
                                messageError();
                            } else if (zSpatialResampling > 5f * maxSpacing) {
                                messageError();
                            } else {
                                messageOK();
                            }

                        } catch (NumberFormatException ex) {
                            zSpatialResampling = 0;
                        }
                        //</editor-fold>
                    }
                } catch (Exception ignore) {
                }
            }

            private void messageOK() {
                /* if (messageGUI != null) {
          messageGUI.end();
        }
        if (FrameGui.getExecutionModel() == ExecutionModel.EXEC_NORMAL) {
          messageGUI = new MessageGui(
                  "Texture message",
                  "Z spatial resampling: " + zSpatialResampling + " mm is valid.",
                  MessageGui.OK_MESSAGE_QUICK
          );
        }*/
            }

            private void messageError() {
                if (messageGUI != null) {
                    messageGUI.end();
                }
                messageGUI = new MessageGui(
                        "Texture message",
                        "Range of Z spatial resampling: " + zSpatialResampling + " mm is not valid.\n"
                        + "Min bound must be: "
                        //+ " >= 0.2*min(spZ, spY, spX)"
                        + " >= " + Utils.d2s(0.2f * minSpacing, 2) + "\n"
                        + "Max bound must be: "
                        // + " <= 5*max(spZ, spY, spX)"
                        + " <= " + Utils.d2s(5f * maxSpacing, 2),
                        MessageGui.WARNING_MESSAGE);
            }

        });
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroupResampling = new javax.swing.ButtonGroup();
        buttonGroupDimensionProcessing = new javax.swing.ButtonGroup();
        jPanelSpatialResampling = new javax.swing.JPanel();
        jPanel4 = new javax.swing.JPanel();
        jPanelSpatialResamplingX = new javax.swing.JPanel();
        jLabelXSpatialResampling = new javax.swing.JLabel();
        jTextFieldXSpatialResampling = new javax.swing.JTextField();
        jPanelSpatialResamplingY = new javax.swing.JPanel();
        jLabelYSpatialResampling = new javax.swing.JLabel();
        jTextFieldYSpatialResampling = new javax.swing.JTextField();
        jPanelSpatialResamplingZ = new javax.swing.JPanel();
        jLabelZSpatialResampling = new javax.swing.JLabel();
        jTextFieldZSpatialResampling = new javax.swing.JTextField();
        jPanelSpatialResamplingDefaultSetting = new javax.swing.JPanel();
        jButtonSpatialResamplingHelp = new javax.swing.JButton();
        jButtonSpatialResamplingDefaultSetting = new javax.swing.JButton();
        jPanelInput = new javax.swing.JPanel();
        jPanelIntensityDiscretization = new javax.swing.JPanel();
        jPanelDefaultSetting = new javax.swing.JPanel();
        jButtonIntensityDiscretizationHelp = new javax.swing.JButton();
        jButtonIntensityDiscretizationDefaultSetting = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jLabelNbGrey = new javax.swing.JLabel();
        jTextFieldNbGrey = new javax.swing.JTextField();
        jPanel6 = new javax.swing.JPanel();
        jLabelBinSize = new javax.swing.JLabel();
        jTextFieldBinSize = new javax.swing.JTextField();
        jPanelIntensityRescaling = new javax.swing.JPanel();
        jPanel7 = new javax.swing.JPanel();
        jPanelAbsolute = new javax.swing.JPanel();
        jRadioButtonAbsoluteMinMax = new javax.swing.JRadioButton();
        jPanelAbsoluteParamtersMinMax = new javax.swing.JPanel();
        jPanelIntensityResamplingDefaultSetting = new javax.swing.JPanel();
        jButtonIntensityResamplingHelp = new javax.swing.JButton();
        jButtonIntensityResamplingDefaultSetting = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jPanelMinBound = new javax.swing.JPanel();
        jLabelMinBound = new javax.swing.JLabel();
        jTextFieldMinBound = new javax.swing.JTextField();
        jPanelMaxBound = new javax.swing.JPanel();
        jLabelMaxBound = new javax.swing.JLabel();
        jTextFieldMaxBound = new javax.swing.JTextField();
        jPanelRelativeRoiMinRoiMax = new javax.swing.JPanel();
        jRadioButtonRelativeMinMax = new javax.swing.JRadioButton();
        jPanelRelativeRoiSd = new javax.swing.JPanel();
        jRadioButtonRelativeSd = new javax.swing.JRadioButton();
        jPanelDimensionProcessing = new javax.swing.JPanel();
        jRadioButtonDimensionProcessing2D = new javax.swing.JRadioButton();
        jRadioButtonDimensionProcessing3D = new javax.swing.JRadioButton();
        jPanelCommand = new javax.swing.JPanel();
        jToggleButtonOutputParameters = new javax.swing.JToggleButton();
        jButtonMenuItemCheckRoiForTexture = new javax.swing.JButton();
        jButtonRun = new javax.swing.JButton();

        setOpaque(false);

        jPanelSpatialResampling.setOpaque(false);
        jPanelSpatialResampling.setLayout(new java.awt.BorderLayout());

        jPanel4.setOpaque(false);
        jPanel4.setLayout(new javax.swing.BoxLayout(jPanel4, javax.swing.BoxLayout.Y_AXIS));

        jPanelSpatialResamplingX.setOpaque(false);
        jPanelSpatialResamplingX.setLayout(new java.awt.BorderLayout());

        jLabelXSpatialResampling.setText("spacing X (mm): ");
        jPanelSpatialResamplingX.add(jLabelXSpatialResampling, java.awt.BorderLayout.WEST);
        jPanelSpatialResamplingX.add(jTextFieldXSpatialResampling, java.awt.BorderLayout.CENTER);

        jPanel4.add(jPanelSpatialResamplingX);

        jPanelSpatialResamplingY.setOpaque(false);
        jPanelSpatialResamplingY.setLayout(new java.awt.BorderLayout());

        jLabelYSpatialResampling.setText("spacing Y (mm): ");
        jPanelSpatialResamplingY.add(jLabelYSpatialResampling, java.awt.BorderLayout.WEST);
        jPanelSpatialResamplingY.add(jTextFieldYSpatialResampling, java.awt.BorderLayout.CENTER);

        jPanel4.add(jPanelSpatialResamplingY);

        jPanelSpatialResamplingZ.setOpaque(false);
        jPanelSpatialResamplingZ.setLayout(new java.awt.BorderLayout());

        jLabelZSpatialResampling.setText("spacing Z (mm): ");
        jPanelSpatialResamplingZ.add(jLabelZSpatialResampling, java.awt.BorderLayout.WEST);
        jPanelSpatialResamplingZ.add(jTextFieldZSpatialResampling, java.awt.BorderLayout.CENTER);

        jPanel4.add(jPanelSpatialResamplingZ);

        jPanelSpatialResampling.add(jPanel4, java.awt.BorderLayout.CENTER);

        jPanelSpatialResamplingDefaultSetting.setOpaque(false);
        jPanelSpatialResamplingDefaultSetting.setLayout(new javax.swing.BoxLayout(jPanelSpatialResamplingDefaultSetting, javax.swing.BoxLayout.Y_AXIS));

        jButtonSpatialResamplingHelp.setMargin(new java.awt.Insets(1, 1, 1, 1));
        jButtonSpatialResamplingHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSpatialResamplingHelpActionPerformed(evt);
            }
        });
        jPanelSpatialResamplingDefaultSetting.add(jButtonSpatialResamplingHelp);

        jButtonSpatialResamplingDefaultSetting.setMargin(new java.awt.Insets(1, 1, 1, 1));
        jButtonSpatialResamplingDefaultSetting.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSpatialResamplingDefaultSettingActionPerformed(evt);
            }
        });
        jPanelSpatialResamplingDefaultSetting.add(jButtonSpatialResamplingDefaultSetting);

        jPanelSpatialResampling.add(jPanelSpatialResamplingDefaultSetting, java.awt.BorderLayout.EAST);

        add(jPanelSpatialResampling);

        jPanelInput.setOpaque(false);

        jPanelIntensityDiscretization.setOpaque(false);
        jPanelIntensityDiscretization.setLayout(new java.awt.BorderLayout());

        jPanelDefaultSetting.setOpaque(false);
        jPanelDefaultSetting.setLayout(new javax.swing.BoxLayout(jPanelDefaultSetting, javax.swing.BoxLayout.Y_AXIS));

        jButtonIntensityDiscretizationHelp.setMargin(new java.awt.Insets(1, 1, 1, 1));
        jButtonIntensityDiscretizationHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityDiscretizationHelpActionPerformed(evt);
            }
        });
        jPanelDefaultSetting.add(jButtonIntensityDiscretizationHelp);

        jButtonIntensityDiscretizationDefaultSetting.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityDiscretizationDefaultSettingActionPerformed(evt);
            }
        });
        jPanelDefaultSetting.add(jButtonIntensityDiscretizationDefaultSetting);

        jPanelIntensityDiscretization.add(jPanelDefaultSetting, java.awt.BorderLayout.EAST);

        jPanel3.setOpaque(false);
        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.Y_AXIS));

        jPanel5.setOpaque(false);
        jPanel5.setLayout(new javax.swing.BoxLayout(jPanel5, javax.swing.BoxLayout.X_AXIS));

        jLabelNbGrey.setText("Nb of grey levels:   ");
        jPanel5.add(jLabelNbGrey);

        jTextFieldNbGrey.setColumns(5);
        jTextFieldNbGrey.setToolTipText("default: 64 bin");
        jTextFieldNbGrey.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldNbGreyKeyReleased(evt);
            }
        });
        jPanel5.add(jTextFieldNbGrey);

        jPanel3.add(jPanel5);

        jPanel6.setOpaque(false);
        jPanel6.setLayout(new javax.swing.BoxLayout(jPanel6, javax.swing.BoxLayout.X_AXIS));

        jLabelBinSize.setText("size of bins:  ");
        jPanel6.add(jLabelBinSize);

        jTextFieldBinSize.setColumns(5);
        jTextFieldBinSize.setToolTipText("default: 64 bin");
        jTextFieldBinSize.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldBinSizeKeyReleased(evt);
            }
        });
        jPanel6.add(jTextFieldBinSize);

        jPanel3.add(jPanel6);

        jPanelIntensityDiscretization.add(jPanel3, java.awt.BorderLayout.CENTER);

        jPanelInput.add(jPanelIntensityDiscretization);

        jPanelIntensityRescaling.setOpaque(false);
        jPanelIntensityRescaling.setLayout(new java.awt.BorderLayout());

        jPanel7.setOpaque(false);
        jPanel7.setLayout(new javax.swing.BoxLayout(jPanel7, javax.swing.BoxLayout.Y_AXIS));

        jPanelAbsolute.setOpaque(false);
        jPanelAbsolute.setLayout(new java.awt.BorderLayout());

        buttonGroupResampling.add(jRadioButtonAbsoluteMinMax);
        jRadioButtonAbsoluteMinMax.setText("absolute (bounds: min<>max)");
        jRadioButtonAbsoluteMinMax.setToolTipText("get min and max bounds defined below");
        jRadioButtonAbsoluteMinMax.setContentAreaFilled(false);
        jRadioButtonAbsoluteMinMax.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jRadioButtonAbsoluteMinMaxItemStateChanged(evt);
            }
        });
        jRadioButtonAbsoluteMinMax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonAbsoluteMinMaxActionPerformed(evt);
            }
        });
        jPanelAbsolute.add(jRadioButtonAbsoluteMinMax, java.awt.BorderLayout.CENTER);

        jPanel7.add(jPanelAbsolute);

        jPanelAbsoluteParamtersMinMax.setOpaque(false);
        jPanelAbsoluteParamtersMinMax.setLayout(new java.awt.BorderLayout());

        jPanelIntensityResamplingDefaultSetting.setOpaque(false);
        jPanelIntensityResamplingDefaultSetting.setLayout(new javax.swing.BoxLayout(jPanelIntensityResamplingDefaultSetting, javax.swing.BoxLayout.Y_AXIS));

        jButtonIntensityResamplingHelp.setMargin(new java.awt.Insets(1, 1, 1, 1));
        jButtonIntensityResamplingHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityResamplingHelpActionPerformed(evt);
            }
        });
        jPanelIntensityResamplingDefaultSetting.add(jButtonIntensityResamplingHelp);

        jButtonIntensityResamplingDefaultSetting.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIntensityResamplingDefaultSettingActionPerformed(evt);
            }
        });
        jPanelIntensityResamplingDefaultSetting.add(jButtonIntensityResamplingDefaultSetting);

        jPanelAbsoluteParamtersMinMax.add(jPanelIntensityResamplingDefaultSetting, java.awt.BorderLayout.EAST);

        jPanel2.setOpaque(false);
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.Y_AXIS));

        jPanelMinBound.setOpaque(false);
        jPanelMinBound.setLayout(new java.awt.BorderLayout());

        jLabelMinBound.setText("min bound:   ");
        jPanelMinBound.add(jLabelMinBound, java.awt.BorderLayout.WEST);

        jTextFieldMinBound.setColumns(5);
        jTextFieldMinBound.setToolTipText("default: 20");
        jTextFieldMinBound.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldMinBoundKeyReleased(evt);
            }
        });
        jPanelMinBound.add(jTextFieldMinBound, java.awt.BorderLayout.CENTER);

        jPanel2.add(jPanelMinBound);

        jPanelMaxBound.setOpaque(false);
        jPanelMaxBound.setLayout(new java.awt.BorderLayout());

        jLabelMaxBound.setText("max bound:   ");
        jPanelMaxBound.add(jLabelMaxBound, java.awt.BorderLayout.WEST);

        jTextFieldMaxBound.setColumns(5);
        jTextFieldMaxBound.setToolTipText("default: 20");
        jTextFieldMaxBound.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTextFieldMaxBoundKeyReleased(evt);
            }
        });
        jPanelMaxBound.add(jTextFieldMaxBound, java.awt.BorderLayout.CENTER);

        jPanel2.add(jPanelMaxBound);

        jPanelAbsoluteParamtersMinMax.add(jPanel2, java.awt.BorderLayout.CENTER);

        jPanel7.add(jPanelAbsoluteParamtersMinMax);

        jPanelRelativeRoiMinRoiMax.setOpaque(false);
        jPanelRelativeRoiMinRoiMax.setLayout(new java.awt.BorderLayout());

        buttonGroupResampling.add(jRadioButtonRelativeMinMax);
        jRadioButtonRelativeMinMax.setText("relative (ROI: min<>max)");
        jRadioButtonRelativeMinMax.setToolTipText("get min and max values of ROI");
        jRadioButtonRelativeMinMax.setContentAreaFilled(false);
        jRadioButtonRelativeMinMax.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jRadioButtonRelativeMinMaxItemStateChanged(evt);
            }
        });
        jRadioButtonRelativeMinMax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonRelativeMinMaxActionPerformed(evt);
            }
        });
        jPanelRelativeRoiMinRoiMax.add(jRadioButtonRelativeMinMax, java.awt.BorderLayout.CENTER);

        jPanel7.add(jPanelRelativeRoiMinRoiMax);

        jPanelRelativeRoiSd.setOpaque(false);
        jPanelRelativeRoiSd.setLayout(new java.awt.BorderLayout());

        buttonGroupResampling.add(jRadioButtonRelativeSd);
        jRadioButtonRelativeSd.setText("relative (ROI: mean+-3sd)");
        jRadioButtonRelativeSd.setToolTipText("get min and max values of ROI");
        jRadioButtonRelativeSd.setContentAreaFilled(false);
        jRadioButtonRelativeSd.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jRadioButtonRelativeSdItemStateChanged(evt);
            }
        });
        jRadioButtonRelativeSd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonRelativeSdActionPerformed(evt);
            }
        });
        jPanelRelativeRoiSd.add(jRadioButtonRelativeSd, java.awt.BorderLayout.CENTER);

        jPanel7.add(jPanelRelativeRoiSd);

        jPanelIntensityRescaling.add(jPanel7, java.awt.BorderLayout.CENTER);

        jPanelInput.add(jPanelIntensityRescaling);

        jPanelDimensionProcessing.setOpaque(false);

        buttonGroupDimensionProcessing.add(jRadioButtonDimensionProcessing2D);
        jRadioButtonDimensionProcessing2D.setText("2D");
        jRadioButtonDimensionProcessing2D.setContentAreaFilled(false);
        jRadioButtonDimensionProcessing2D.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonDimensionProcessing2DActionPerformed(evt);
            }
        });
        jPanelDimensionProcessing.add(jRadioButtonDimensionProcessing2D);

        buttonGroupDimensionProcessing.add(jRadioButtonDimensionProcessing3D);
        jRadioButtonDimensionProcessing3D.setSelected(true);
        jRadioButtonDimensionProcessing3D.setText("3D");
        jRadioButtonDimensionProcessing3D.setContentAreaFilled(false);
        jRadioButtonDimensionProcessing3D.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonDimensionProcessing3DActionPerformed(evt);
            }
        });
        jPanelDimensionProcessing.add(jRadioButtonDimensionProcessing3D);

        jPanelInput.add(jPanelDimensionProcessing);

        add(jPanelInput);

        jPanelCommand.setOpaque(false);

        jToggleButtonOutputParameters.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonOutputParametersActionPerformed(evt);
            }
        });
        jPanelCommand.add(jToggleButtonOutputParameters);

        jButtonMenuItemCheckRoiForTexture.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonMenuItemCheckRoiForTextureActionPerformed(evt);
            }
        });
        jPanelCommand.add(jButtonMenuItemCheckRoiForTexture);

        jButtonRun.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRunActionPerformed(evt);
            }
        });
        jPanelCommand.add(jButtonRun);

        add(jPanelCommand);
    }// </editor-fold>//GEN-END:initComponents


    private void jTextFieldNbGreyKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldNbGreyKeyReleased

        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
        } catch (NumberFormatException ex) {
            maxBound = 0;
        }

        try {
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());
        } catch (NumberFormatException ex) {
            minBound = 0;
        }

        try {
            nbGreyLevels = Float.parseFloat(this.jTextFieldNbGrey.getText());
        } catch (NumberFormatException ex) {
            nbGreyLevels = 0;
        }

        try {
            binSize = Float.parseFloat(this.jTextFieldBinSize.getText());
        } catch (NumberFormatException ex) {
            binSize = 0;
        }

        checkIntensityDiscretization_with_jTextFieldNbGreyKeyReleased();
        saveProperties();
    }//GEN-LAST:event_jTextFieldNbGreyKeyReleased

    private void checkIntensityDiscretization_with_jTextFieldNbGreyKeyReleased() {
        if (this.jRadioButtonAbsoluteMinMax.isSelected()) {
            binSize = (maxBound - minBound) / (nbGreyLevels - 1);
            this.jTextFieldBinSize.setText(String.valueOf(binSize));
            this.jTextFieldBinSize.setCaretPosition(0);

            buttonAbsolute = true;
            buttonRelativeMinMax = false;
            buttonRelativeMeanSd = false;
        } else if (this.jRadioButtonRelativeMinMax.isSelected()) {
            binSize = 0;
            this.jTextFieldBinSize.setText("---");

            buttonAbsolute = false;
            buttonRelativeMinMax = true;
            buttonRelativeMeanSd = false;
        } else if (this.jRadioButtonRelativeSd.isSelected()) {
            binSize = 0;
            this.jTextFieldBinSize.setText("---");

            buttonAbsolute = false;
            buttonRelativeMinMax = false;
            buttonRelativeMeanSd = true;
        }
    }

    private void checkIntensityDiscretization_with_jTextFieldBinSizeKeyReleased() {
        if (this.jRadioButtonAbsoluteMinMax.isSelected()) {
            nbGreyLevels = (int) ((maxBound - minBound) / binSize) + 1;
            this.jTextFieldNbGrey.setText(String.valueOf(nbGreyLevels));
            this.jTextFieldNbGrey.setCaretPosition(0);

        } else if (this.jRadioButtonRelativeMinMax.isSelected()) {
            nbGreyLevels = 0;
            this.jTextFieldNbGrey.setText("---");

        } else if (this.jRadioButtonRelativeSd.isSelected()) {
            nbGreyLevels = 0;
            this.jTextFieldNbGrey.setText("---");
        }
    }

    private void jTextFieldBinSizeKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldBinSizeKeyReleased

        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
        } catch (NumberFormatException ex) {
            maxBound = 0;
        }

        try {
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());
        } catch (NumberFormatException ex) {
            minBound = 0;
        }

        try {
            nbGreyLevels = Float.parseFloat(this.jTextFieldNbGrey.getText());
        } catch (NumberFormatException ex) {
            nbGreyLevels = 0;
        }

        try {
            binSize = Float.parseFloat(this.jTextFieldBinSize.getText());
        } catch (NumberFormatException ex) {
            binSize = 0;
        }

        checkIntensityDiscretization_with_jTextFieldBinSizeKeyReleased();
        saveProperties();
    }//GEN-LAST:event_jTextFieldBinSizeKeyReleased

    private void jTextFieldMinBoundKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldMinBoundKeyReleased
        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());

            if (minBound >= maxBound) {
                MessageGui messageGUI = new MessageGui(
                        "Texture message",
                        "Range of bounds is not valid.",
                        MessageGui.WARNING_MESSAGE);
            }
            nbGreyLevels = Float.parseFloat(this.jTextFieldNbGrey.getText());
            binSize = (maxBound - minBound) / (nbGreyLevels - 1);
            this.jTextFieldBinSize.setText(String.valueOf(binSize));
            this.jTextFieldBinSize.setCaretPosition(0);
            saveProperties();

        } catch (Exception ignore) {
        }
    }//GEN-LAST:event_jTextFieldMinBoundKeyReleased

    private void jTextFieldMaxBoundKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldMaxBoundKeyReleased
        try {
            maxBound = Float.parseFloat(this.jTextFieldMaxBound.getText());
            minBound = Float.parseFloat(this.jTextFieldMinBound.getText());
            if (minBound >= maxBound) {
                MessageGui messageGUI = new MessageGui(
                        "Texture message",
                        "Range of bounds is not valid.",
                        MessageGui.WARNING_MESSAGE);
            }
            nbGreyLevels = Float.parseFloat(this.jTextFieldNbGrey.getText());
            binSize = (maxBound - minBound) / (nbGreyLevels - 1);
            this.jTextFieldBinSize.setText(String.valueOf(binSize));
            this.jTextFieldBinSize.setCaretPosition(0);
            saveProperties();

        } catch (Exception ignore) {
        }
    }//GEN-LAST:event_jTextFieldMaxBoundKeyReleased

    private void jRadioButtonRelativeMinMaxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jRadioButtonRelativeMinMaxItemStateChanged
        this.jLabelMinBound.setEnabled(false);
        this.jTextFieldMinBound.setEnabled(false);
        this.jLabelMaxBound.setEnabled(false);
        this.jTextFieldMaxBound.setEnabled(false);
    }//GEN-LAST:event_jRadioButtonRelativeMinMaxItemStateChanged

    private void jRadioButtonAbsoluteMinMaxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jRadioButtonAbsoluteMinMaxItemStateChanged
        this.jLabelMinBound.setEnabled(true);
        this.jTextFieldMinBound.setEnabled(true);
        this.jLabelMaxBound.setEnabled(true);
        this.jTextFieldMaxBound.setEnabled(true);
    }//GEN-LAST:event_jRadioButtonAbsoluteMinMaxItemStateChanged

    private void jButtonRunActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRunActionPerformed
        if (series == null) {
            if (FrameGui.session.getSeriesCount() == 1) {
                series = FrameController.seriesRef;
            } else {
                new MessageGui(
                        "Texture",
                        "No selected series found. Operation aborted.",
                        MessageGui.WARNING_MESSAGE
                );
                return;
            }
        }
        saveProperties();
        FrameController.setActivatedSeriesOnAllFrames(series);
        if (FrameGui.protocolFrame.getTypeRepaint() == ProtocolFrame.HORIZONTAL_REPAINT) {
            FrameGui.protocolFrame.texBoxOption.setCloseOption();
        }
        (new TextureController(series, null, null)).start(); // null properties, data given by GUI
    }//GEN-LAST:event_jButtonRunActionPerformed

    private void jToggleButtonOutputParametersActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonOutputParametersActionPerformed
        texJFrameOutputParameters.setVisible(jToggleButtonOutputParameters.isSelected());
    }//GEN-LAST:event_jToggleButtonOutputParametersActionPerformed

    private void jRadioButtonRelativeSdItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jRadioButtonRelativeSdItemStateChanged
        this.jLabelMinBound.setEnabled(false);
        this.jTextFieldMinBound.setEnabled(false);
        this.jLabelMaxBound.setEnabled(false);
        this.jTextFieldMaxBound.setEnabled(false);
    }//GEN-LAST:event_jRadioButtonRelativeSdItemStateChanged

    private void jButtonIntensityDiscretizationDefaultSettingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityDiscretizationDefaultSettingActionPerformed
        this.setIntensityResamplingDefaultValues();
        this.setAllFieldValues();
        MessageGui messageGUI = new MessageGui(
                "Texture message",
                "Default settings were applied",
                MessageGui.OK_MESSAGE
        );
    }//GEN-LAST:event_jButtonIntensityDiscretizationDefaultSettingActionPerformed

    private void jButtonIntensityResamplingDefaultSettingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityResamplingDefaultSettingActionPerformed
        this.setIntensityResamplingDefaultValues();
        this.setAllFieldValues();
        MessageGui messageGUI = new MessageGui(
                "Texture message",
                "Default settings were applied",
                MessageGui.OK_MESSAGE
        );
    }//GEN-LAST:event_jButtonIntensityResamplingDefaultSettingActionPerformed

    private void jButtonSpatialResamplingDefaultSettingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSpatialResamplingDefaultSettingActionPerformed
        this.setSpatialResamplingDefaultValues();
        this.setAllFieldValues();
        MessageGui messageGUI = new MessageGui(
                "Texture message",
                "Default settings from native voxels were applied",
                MessageGui.OK_MESSAGE
        );
    }//GEN-LAST:event_jButtonSpatialResamplingDefaultSettingActionPerformed

    private void jButtonSpatialResamplingHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSpatialResamplingHelpActionPerformed
        MessageGui messageGUI = new MessageGui(
                "Spatial Resampling",
                "As most textural feature values depend on the voxel size,"
                + " please select the voxel size to be used for calculation"
                + " if you want to compare your values with values published by others.",
                MessageGui.OK_MESSAGE
        );
    }//GEN-LAST:event_jButtonSpatialResamplingHelpActionPerformed

    private void jButtonIntensityResamplingHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityResamplingHelpActionPerformed
        MessageGui messageGUI = new MessageGui(
                "Intensity Rescaling",
                "In PET, AD is preferred, between 0 and the max SUV you expect in the VOI in your whole cohort.",
                MessageGui.OK_MESSAGE
        );
    }//GEN-LAST:event_jButtonIntensityResamplingHelpActionPerformed

    private void jButtonIntensityDiscretizationHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIntensityDiscretizationHelpActionPerformed
        MessageGui messageGUI = new MessageGui(
                "Intensity Discretization",
                "In PET, 64 levels are usually appropriate.", // toolTipText
                MessageGui.OK_MESSAGE
        );
    }//GEN-LAST:event_jButtonIntensityDiscretizationHelpActionPerformed

    private void jRadioButtonRelativeMinMaxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonRelativeMinMaxActionPerformed
        jTextFieldNbGreyKeyReleased(null);
    }//GEN-LAST:event_jRadioButtonRelativeMinMaxActionPerformed

    private void jRadioButtonRelativeSdActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonRelativeSdActionPerformed
        jTextFieldNbGreyKeyReleased(null);
    }//GEN-LAST:event_jRadioButtonRelativeSdActionPerformed

    private void jRadioButtonAbsoluteMinMaxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonAbsoluteMinMaxActionPerformed
        jTextFieldNbGreyKeyReleased(null);
    }//GEN-LAST:event_jRadioButtonAbsoluteMinMaxActionPerformed

    /**
     * action performed to verification of check list parameters to process
     * texture protocol with success
     * <br>applied on RoiManager.roiMainActived
     * <br>it's interesting when you will make a lot of ROI for texture script
     * processing
     */
  private void jButtonMenuItemCheckRoiForTextureActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonMenuItemCheckRoiForTextureActionPerformed

      //<editor-fold defaultstate="collapsed" desc="check if no selection : abort procedure">
      if (RoiController.roiMainActived == null) {
          MessageGui messageGUI = new MessageGui(
                  "No selection",
                  "Select a ROI and start again",
                  MessageGui.WARNING_MESSAGE);
          return;
      }
      //</editor-fold>

      // initialization of setting of result messageGUI
      String titleMessage = RoiController.roiMainActived.roiModel.title + " for texture";
      String contentMessage = "";
      int typeMessage = MessageGui.OK_MESSAGE;

      this.jTextFieldZSpatialResampling.getText();
      
      //<editor-fold defaultstate="collapsed" desc="check size of ROI">
      CoorRange rangeVoxelBounds = RoiController.roiZYX.getBounds(RoiController.roiMainActived.roiModel.index);
      float nbVoxelMin = rangeVoxelBounds.max.getZ() - rangeVoxelBounds.min.getZ() > 1
              ? TextureControllerSub.ROI_VOX_MIN_3D
              : TextureControllerSub.ROI_VOX_MIN_2D;

      SpModel sp = series.rawTZYX.iopIppSp.spModel;
      double nbResamplingVoxels = RoiController.roiMainActived.roiModel.sizeVoxel
              * sp.getZSpacing() / zSpatialResampling
              * sp.getYSpacing() / ySpatialResampling
              * sp.getXSpacing() / xSpatialResampling;
      
      if (nbResamplingVoxels < nbVoxelMin) {
          contentMessage += "ROI  size:\n"
                            + " =" + Utils.d2s(nbResamplingVoxels,0)+"vx is too small\n"
                            + " ->must be greater or equals to " + nbVoxelMin + " voxels after resampling\n";
          typeMessage = MessageGui.ERROR_MESSAGE;
      } else {
          contentMessage += "ROI size:\n"
                  + " =" + Utils.d2s(RoiController.roiMainActived.roiModel.sizeVoxel, 0) +"vx before resampling\n"
                  + " =" + Utils.d2s(nbResamplingVoxels, 0) +"vx after resampling -> OK\n";
      }
      //</editor-fold>

      contentMessage += "\n";
      
      //<editor-fold defaultstate="collapsed" desc="check number of clusters">
      Split3DMainSub picROISplit = new Split3DMainSub(RoiController.roiMainActived, /*messageGUI*/ null);
      int nbCluster = picROISplit.checkNumberOfCluster();
      if (nbCluster > 1) {
          contentMessage += "ROI cluster:\n"
                            + " =" + nbCluster + " clusters \n"
                            + " -> must be only one cluster\n";
          typeMessage = MessageGui.ERROR_MESSAGE;
      } else {
          contentMessage += "ROI cluster:\n"
                            + " =One cluster in ROI: OK\n";
      }
      //</editor-fold>

      contentMessage += typeMessage == MessageGui.OK_MESSAGE
              ? "\n-> Check list has been successful"
              : "\n-> An error has occurred";

      MessageGui messageGUI = new MessageGui(titleMessage, contentMessage, typeMessage);
  }//GEN-LAST:event_jButtonMenuItemCheckRoiForTextureActionPerformed

    private void jRadioButtonDimensionProcessing2DActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonDimensionProcessing2DActionPerformed
        this.dimensionProcessing = "2D";
    }//GEN-LAST:event_jRadioButtonDimensionProcessing2DActionPerformed

    private void jRadioButtonDimensionProcessing3DActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonDimensionProcessing3DActionPerformed
        this.dimensionProcessing = "3D";
    }//GEN-LAST:event_jRadioButtonDimensionProcessing3DActionPerformed

    /**
     * get Intensity Rescaling MaxBound if number format in field is correct,
     * else return 0
     *
     * @return
     */
    public float getIntensityRescalingMaxBound() {
        return this.maxBound == NO_DEF ? 0f : this.maxBound;
    }

    /**
     * get Intensity Rescaling MinBound if number format in field is correct,
     * else return 0
     *
     * @return
     */
    public float getIntensityRescalingMinBound() {
        return this.minBound == NO_DEF ? 0f : this.minBound;

    }

    /**
     * get Intensity Discretization NbGreyLevels if number format in field is
     * correct, else return 0
     *
     * @return
     */
    public float getIntensityDiscretizationNbGreyLevels() {
        return this.nbGreyLevels == NO_DEF ? 0f : this.nbGreyLevels;
    }

    /**
     * get Intensity Discretization BinSize if number format in field is
     * correct, else return 0
     *
     * @return
     */
    public float getIntensityDiscretizationBinSize() {
        return this.binSize == NO_DEF ? 0f : this.binSize;
    }

    /**
     * get spatial resampling in size x (mm)
     *
     * @return
     */
    public double getXSpatialResampling() {
        return xSpatialResampling;
    }

    /**
     * get spatial resampling in size y (mm)
     *
     * @return
     */
    public double getYSpatialResampling() {
        return ySpatialResampling;
    }

    /**
     * get spatial resampling in size z (mm)
     *
     * @return
     */
    public double getZSpatialResampling() {
        return zSpatialResampling;
    }

    /**
     *
     * save properties in prefsOption and in properties file
     */
    private void saveProperties() {
        prefsOption.setFloat("LIFEx.Texture.NbGrey", nbGreyLevels);
        prefsOption.setFloat("LIFEx.Texture.BinSize", binSize);

        prefsOption.setFloat("LIFEx.Texture.MinBound", minBound);
        prefsOption.setFloat("LIFEx.Texture.MaxBound", maxBound);

        prefsOption.setBoolean("LIFEx.Texture.ButtonAbsolute", buttonAbsolute);
        prefsOption.setBoolean("LIFEx.Texture.ButtonRelativeMinMax", buttonRelativeMinMax);
        prefsOption.setBoolean("LIFEx.Texture.ButtonRelativeMeanSd", buttonRelativeMeanSd);

        prefsOption.setString("LIFEx.Texture.DimensionProcessing", dimensionProcessing);

        prefsOption.setDouble("LIFEx.Texture.ZSpatialResampling", zSpatialResampling);
        prefsOption.setDouble("LIFEx.Texture.YSpatialResampling", ySpatialResampling);
        prefsOption.setDouble("LIFEx.Texture.XSpatialResampling", xSpatialResampling);

        prefsOption.saveProperties(filenameProperties);
    }

    /**
     * load properties from properties file named filenameProperties
     * <br>if file properties is not exist, setting values with default values
     *
     * @return true if loading success, false otherwise
     */
    private boolean loadProperties() {
        if (prefsOption.loadProperties(filenameProperties)) {
            //<editor-fold defaultstate="collapsed" desc="setting values with properties from file">
            nbGreyLevels = prefsOption.getFloat("LIFEx.Texture.NbGrey", -1);
            binSize = prefsOption.getFloat("LIFEx.Texture.BinSize", -1);

            minBound = prefsOption.getFloat("LIFEx.Texture.MinBound", -1);
            maxBound = prefsOption.getFloat("LIFEx.Texture.MaxBound", -1);

            buttonAbsolute = prefsOption.getBoolean("LIFEx.Texture.ButtonAbsolute", true);
            buttonRelativeMinMax = prefsOption.getBoolean("LIFEx.Texture.ButtonRelativeMinMax", false);
            buttonRelativeMeanSd = prefsOption.getBoolean("LIFEx.Texture.ButtonRelativeMeanSd", false);

            dimensionProcessing = Settings.getString("LIFEx.Texture.DimensionProcessing", "3D");

            zSpatialResampling = prefsOption.getFloat("LIFEx.Texture.ZSpatialResampling", -1);
            ySpatialResampling = prefsOption.getFloat("LIFEx.Texture.YSpatialResampling", -1);
            xSpatialResampling = prefsOption.getFloat("LIFEx.Texture.XSpatialResampling", -1);
            //</editor-fold>
            return true;
        } else {
            //<editor-fold defaultstate="collapsed" desc="setting values with default values">
            nbGreyLevels = NO_DEF;
            binSize = NO_DEF;

            minBound = NO_DEF;
            maxBound = NO_DEF;

            buttonAbsolute = true;
            buttonRelativeMinMax = false;
            buttonRelativeMeanSd = false;

            dimensionProcessing = "3D";
            
            zSpatialResampling = -1;
            ySpatialResampling = -1;
            xSpatialResampling = -1;
            //</editor-fold>
            return false;
        }
    }

    public float nbGreyLevels;
    public float binSize;

    public float minBound;
    public float maxBound;

    public boolean buttonAbsolute;
    public boolean buttonRelativeMinMax;
    public boolean buttonRelativeMeanSd;

    public double zSpatialResampling;
    public double ySpatialResampling;
    public double xSpatialResampling;

    public String dimensionProcessing;
    
    public String scriptingCsv = Settings.getString("LIFEx.Texture.SessionCsv", "");

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupDimensionProcessing;
    private javax.swing.ButtonGroup buttonGroupResampling;
    private javax.swing.JButton jButtonIntensityDiscretizationDefaultSetting;
    private javax.swing.JButton jButtonIntensityDiscretizationHelp;
    private javax.swing.JButton jButtonIntensityResamplingDefaultSetting;
    private javax.swing.JButton jButtonIntensityResamplingHelp;
    protected javax.swing.JButton jButtonMenuItemCheckRoiForTexture;
    protected javax.swing.JButton jButtonRun;
    private javax.swing.JButton jButtonSpatialResamplingDefaultSetting;
    private javax.swing.JButton jButtonSpatialResamplingHelp;
    private javax.swing.JLabel jLabelBinSize;
    private javax.swing.JLabel jLabelMaxBound;
    private javax.swing.JLabel jLabelMinBound;
    private javax.swing.JLabel jLabelNbGrey;
    private javax.swing.JLabel jLabelXSpatialResampling;
    private javax.swing.JLabel jLabelYSpatialResampling;
    private javax.swing.JLabel jLabelZSpatialResampling;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanelAbsolute;
    private javax.swing.JPanel jPanelAbsoluteParamtersMinMax;
    private javax.swing.JPanel jPanelCommand;
    private javax.swing.JPanel jPanelDefaultSetting;
    private javax.swing.JPanel jPanelDimensionProcessing;
    private javax.swing.JPanel jPanelInput;
    private javax.swing.JPanel jPanelIntensityDiscretization;
    private javax.swing.JPanel jPanelIntensityResamplingDefaultSetting;
    private javax.swing.JPanel jPanelIntensityRescaling;
    private javax.swing.JPanel jPanelMaxBound;
    private javax.swing.JPanel jPanelMinBound;
    private javax.swing.JPanel jPanelRelativeRoiMinRoiMax;
    private javax.swing.JPanel jPanelRelativeRoiSd;
    private javax.swing.JPanel jPanelSpatialResampling;
    private javax.swing.JPanel jPanelSpatialResamplingDefaultSetting;
    private javax.swing.JPanel jPanelSpatialResamplingX;
    private javax.swing.JPanel jPanelSpatialResamplingY;
    private javax.swing.JPanel jPanelSpatialResamplingZ;
    public javax.swing.JRadioButton jRadioButtonAbsoluteMinMax;
    private javax.swing.JRadioButton jRadioButtonDimensionProcessing2D;
    private javax.swing.JRadioButton jRadioButtonDimensionProcessing3D;
    public javax.swing.JRadioButton jRadioButtonRelativeMinMax;
    public javax.swing.JRadioButton jRadioButtonRelativeSd;
    public javax.swing.JTextField jTextFieldBinSize;
    public javax.swing.JTextField jTextFieldMaxBound;
    public javax.swing.JTextField jTextFieldMinBound;
    public javax.swing.JTextField jTextFieldNbGrey;
    private javax.swing.JTextField jTextFieldXSpatialResampling;
    private javax.swing.JTextField jTextFieldYSpatialResampling;
    private javax.swing.JTextField jTextFieldZSpatialResampling;
    public javax.swing.JToggleButton jToggleButtonOutputParameters;
    // End of variables declaration//GEN-END:variables

    public TextureJFrameOutputParameters texJFrameOutputParameters;
    private SeriesModel series;

    private final SettingsOption prefsOption;
    private String filenameProperties;
    static private final float NO_DEF = -99999f;
    static private final String NO_DEF_STRING = "No def?";
}
