/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.histo;

import bardp.gui.FrameGui;
import bardp.img.picChart.PicHistogramGUI;
import bardp.img.controller.FrameController;
import bardp.roi.controller.RoiMain;
import bardp.roi.controller.RoiController;
import bardp.roi.controller.RoiModel;
import java.awt.Color;
import java.util.ArrayList;
import java.util.logging.Level;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.title.Title;
import org.jfree.data.statistics.HistogramDataset;
import org.jfree.data.statistics.HistogramType;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class HistogramAllSeriesGui extends PicHistogramGUI {

    public HistogramAllSeriesGui() {
        histogramDataset = new HistogramDataset();
        histogramDataset.setType(HistogramType.FREQUENCY);
    }

    /**
     * add dataset on allSeries histogram
     *
     * @param roiModel
     * @param rawZYX
     * @param intensityResamplingMinBound
     * @param intensityResamplingMaxBound
     */
    public void addDataset(
            final RoiModel roiModel,
            final double[] rawZYX,
            final double intensityResamplingMinBound,
            final double intensityResamplingMaxBound
    ) {

        try {
            double[] rawZYXdouble = new double[rawZYX.length];
            for (int i = 0; i < rawZYXdouble.length; i++) {
                rawZYXdouble[i] = rawZYX[i];
            }

            int intensityDiscretizationNbGreyLevels = (int) roiModel.texControllerSub.intensityDiscretizationNbGreyLevels;
            if (intensityDiscretizationNbGreyLevels < 1) {
                intensityDiscretizationNbGreyLevels = (int) 1;
            }
            histogramDataset.addSeries(
                    roiModel.title,
                    rawZYXdouble,
                    intensityDiscretizationNbGreyLevels,
                    intensityResamplingMinBound,
                    intensityResamplingMaxBound
            );
        } catch (NullPointerException Ex) {
            FrameGui.LOGGER.log(
                    Level.SEVERE,
                    null,
                    "HistogramAllSeriesGUI.addDataset() : hostValuesFloat=null"
            );
        }
    }

    public void setHistogramVisible() {

        //<editor-fold defaultstate="collapsed" desc="PicHistogramGUI">
        this.jFreeChart = ChartFactory.createHistogram("Histogram",
                FrameController.seriesSelected.seriesUnit.unitY.getLongDescription(), "number #",
                histogramDataset, PlotOrientation.VERTICAL, true, true, false);
        this.jFreeChart.setBackgroundPaint(bardp.ressources.Res.COLOR_BACKGROUND1);
        this.chartPanel = new ChartPanel(this.jFreeChart);
        this.getContentPane().add(this.chartPanel, null);
        LegendTitle legendTitle = this.jFreeChart.getLegend(); // against bug of SubTitle
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="setSubtitles(seriesDescription)">
        ArrayList<Title> subTitles = new ArrayList<>();
        subTitles.add(new TextTitle(FrameController.seriesSelected.seriesDescription));
        this.jFreeChart.setSubtitles(subTitles);
        this.jFreeChart.addLegend(legendTitle); // against bug of SubTitle
        //</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="setColor()">
        XYBarRenderer renderer = (XYBarRenderer) this.jFreeChart.getXYPlot().getRenderer();
        int nbSeries = 0;
        for (RoiMain pROI : RoiController.roiMainList) {
            Color color = new Color(
                    pROI.roiModel.color.getRed(),
                    pROI.roiModel.color.getGreen(),
                    pROI.roiModel.color.getBlue(),
                    128 // half transparency
            );
            renderer.setSeriesPaint(nbSeries, color);
            nbSeries++;
        }
        //</editor-fold>

        this.pack();
        this.setVisible(true);
    }

    public HistogramDataset getHistogramDataset() {
        return histogramDataset;
    }

    private final HistogramDataset histogramDataset;
}
