/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS.  IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE, MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), a public scientific, technical and industrial establishment, having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.shape;

import bardp.raw.RawSparseZYX;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: </p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @from java converstion from pyRadiomics\radiomics\shape.py
 * @version 6.0
 */
public class Shape {

    public double calculateSurfaceArea(
            final RawSparseZYX rawTZYX
    ) {
        double surfaceArea = 0d;

        rawTZYXWithPadding0 = rawTZYX.createCopyWitPadding0();

        zSp = rawTZYXWithPadding0.iopIppSp.spModel.getZSpacing();
        ySp = rawTZYXWithPadding0.iopIppSp.spModel.getYSpacing();
        xSp = rawTZYXWithPadding0.iopIppSp.spModel.getXSpacing();

        // define relative locations of the 8 voxels of a sampling cube
        final int[][] gridAngles = new int[][]{
            {0, 0, 0}, // 0
            {0, 0, 1}, // 1
            {0, 1, 1}, // 2
            {0, 1, 0}, // 3
            {1, 0, 0}, // 4
            {1, 0, 1}, // 5
            {1, 1, 1}, // 6
            {1, 1, 0} // 7
        };

        //<editor-fold defaultstate="collapsed" desc="define variables">
        int i;
        int iGridAngles;
        int iz, iy, ix;
        double az, ay, ax;
        double bz, by, bx;
        double cz, cy, cx;

        int iCube;
        int[][] gridCell = new int[gridAngles.length][3];
        double vertList[][] = new double[12][3];
        int[][] triangles;
        int[] triangle;
        //</editor-fold>

        // iterate over all voxels which may border segmentation or are a part of it
        for (iz = 0; iz < rawTZYXWithPadding0.nz - 1; iz++) { // -1 mandatory, see gridAngles with +1

            for (iy = 0; iy < rawTZYXWithPadding0.ny - 1; iy++) {
                for (ix = 0; ix < rawTZYXWithPadding0.nx - 1; ix++) {
                    // generate lookup index for current cube
                    for (iGridAngles = 0; iGridAngles < gridAngles.length; iGridAngles++) {
                        // indices to corners of current sampling cube
                        gridCell[iGridAngles][0] = gridAngles[iGridAngles][0] + iz;
                        gridCell[iGridAngles][1] = gridAngles[iGridAngles][1] + iy;
                        gridCell[iGridAngles][2] = gridAngles[iGridAngles][2] + ix;
                    }

                    iCube = 0;
                    for (iGridAngles = 0; iGridAngles < gridAngles.length; iGridAngles++) {
                        if (rawTZYXWithPadding0.getZYX(
                                gridCell[iGridAngles][0],
                                gridCell[iGridAngles][1],
                                gridCell[iGridAngles][2]) == 0) {
                            iCube |= 1 << iGridAngles;
                        }
                    }

                    //if (iz==0 && iy==0) System.out.println(cube_idx);
                    // full lookup tables are symmetrical, if cube_idx >= 128, take the XOR,
                    // this allows for lookup tables to be half the size.
                    if (iCube >= 128) {
                        iCube = iCube ^ 0xff; // lookup which edges contain vertices and calculate vertices coordinates
                    }

                    if (edgeTable[iCube] == 0) {
                        continue;
                    }

                    if ((edgeTable[iCube] & 1) == 1) {
                        vertList[0] = interpolate(gridCell, 0, 1);
                    }
                    if ((edgeTable[iCube] & 2) == 2) {
                        vertList[1] = interpolate(gridCell, 1, 2);
                    }
                    if ((edgeTable[iCube] & 4) == 4) {
                        vertList[2] = interpolate(gridCell, 2, 3);
                    }
                    if ((edgeTable[iCube] & 8) == 8) {
                        vertList[3] = interpolate(gridCell, 3, 0);
                    }
                    if ((edgeTable[iCube] & 16) == 16) {
                        vertList[4] = interpolate(gridCell, 4, 5);
                    }
                    if ((edgeTable[iCube] & 32) == 32) {
                        vertList[5] = interpolate(gridCell, 5, 6);
                    }
                    if ((edgeTable[iCube] & 64) == 64) {
                        vertList[6] = interpolate(gridCell, 6, 7);
                    }
                    if ((edgeTable[iCube] & 128) == 128) {
                        vertList[7] = interpolate(gridCell, 7, 4);
                    }
                    if ((edgeTable[iCube] & 256) == 256) {
                        vertList[8] = interpolate(gridCell, 0, 4);
                    }
                    if ((edgeTable[iCube] & 512) == 512) {
                        vertList[9] = interpolate(gridCell, 1, 5);
                    }
                    if ((edgeTable[iCube] & 1024) == 1024) {
                        vertList[10] = interpolate(gridCell, 2, 6);
                    }
                    if ((edgeTable[iCube] & 2048) == 2048) {
                        vertList[11] = interpolate(gridCell, 3, 7);
                    }

                    // calculate triangles
                    triangles = triTable[iCube];
                    for (i = 0; i < triangles.length; i++) {
                        triangle = triangles[i];
                        az = vertList[triangle[1]][0] - vertList[triangle[0]][0];
                        ay = vertList[triangle[1]][1] - vertList[triangle[0]][1];
                        ax = vertList[triangle[1]][2] - vertList[triangle[0]][2];

                        bz = vertList[triangle[2]][0] - vertList[triangle[0]][0];
                        by = vertList[triangle[2]][1] - vertList[triangle[0]][1];
                        bx = vertList[triangle[2]][2] - vertList[triangle[0]][2];

                        cz = ax * by - ay * bx;
                        cy = az * bx - ax * bz;
                        cx = ay * bz - az * by;

                        surfaceArea += .5 * StrictMath.sqrt(
                                StrictMath.pow(cz, 2)
                                + StrictMath.pow(cy, 2)
                                + StrictMath.pow(cx, 2)
                        );
                    }
                }
            }
        }
        return surfaceArea;
    }

    private double[] interpolate(
            final int[][] gridCell,
            int p1,
            int p2
    ) {
        final float p1Value = rawTZYXWithPadding0.getZYX(gridCell[p1][0], gridCell[p1][1], gridCell[p1][2]);
        final float p2Value = rawTZYXWithPadding0.getZYX(gridCell[p2][0], gridCell[p2][1], gridCell[p2][2]);
        final double diff = (0.5 - p1Value) / (p2Value - p1Value);
        return new double[]{
            (double) (gridCell[p1][0] + ((gridCell[p2][0] - gridCell[p1][0]) * diff)) * zSp,
            (double) (gridCell[p1][1] + ((gridCell[p2][1] - gridCell[p1][1]) * diff)) * ySp,
            (double) (gridCell[p1][2] + ((gridCell[p2][2] - gridCell[p1][2]) * diff)) * xSp
        };
    }

    //<editor-fold defaultstate="collapsed" desc="edgeTable">
    private final int[] edgeTable = new int[]{
        0x0, 0x109, 0x203, 0x30a, 0x406, 0x50f, 0x605, 0x70c,
        0x80c, 0x905, 0xa0f, 0xb06, 0xc0a, 0xd03, 0xe09, 0xf00,
        0x190, 0x99, 0x393, 0x29a, 0x596, 0x49f, 0x795, 0x69c,
        0x99c, 0x895, 0xb9f, 0xa96, 0xd9a, 0xc93, 0xf99, 0xe90,
        0x230, 0x339, 0x33, 0x13a, 0x636, 0x73f, 0x435, 0x53c,
        0xa3c, 0xb35, 0x83f, 0x936, 0xe3a, 0xf33, 0xc39, 0xd30,
        0x3a0, 0x2a9, 0x1a3, 0xaa, 0x7a6, 0x6af, 0x5a5, 0x4ac,
        0xbac, 0xaa5, 0x9af, 0x8a6, 0xfaa, 0xea3, 0xda9, 0xca0,
        0x460, 0x569, 0x663, 0x76a, 0x66, 0x16f, 0x265, 0x36c,
        0xc6c, 0xd65, 0xe6f, 0xf66, 0x86a, 0x963, 0xa69, 0xb60,
        0x5f0, 0x4f9, 0x7f3, 0x6fa, 0x1f6, 0xff, 0x3f5, 0x2fc,
        0xdfc, 0xcf5, 0xfff, 0xef6, 0x9fa, 0x8f3, 0xbf9, 0xaf0,
        0x650, 0x759, 0x453, 0x55a, 0x256, 0x35f, 0x55, 0x15c,
        0xe5c, 0xf55, 0xc5f, 0xd56, 0xa5a, 0xb53, 0x859, 0x950,
        0x7c0, 0x6c9, 0x5c3, 0x4ca, 0x3c6, 0x2cf, 0x1c5, 0xcc,
        0xfcc, 0xec5, 0xdcf, 0xcc6, 0xbca, 0xac3, 0x9c9, 0x8c0
    };
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="triTable">
    private final int[][][] triTable = new int[][][]{
        {{}},
        {{0, 8, 3}},
        {{0, 1, 9}},
        {{1, 8, 3}, {9, 8, 1}},
        {{1, 2, 10}},
        {{0, 8, 3}, {1, 2, 10}},
        {{9, 2, 10}, {0, 2, 9}},
        {{2, 8, 3}, {2, 10, 8}, {10, 9, 8}},
        {{3, 11, 2}},
        {{0, 11, 2}, {8, 11, 0}},
        {{1, 9, 0}, {2, 3, 11}},
        {{1, 11, 2}, {1, 9, 11}, {9, 8, 11}},
        {{3, 10, 1}, {11, 10, 3}},
        {{0, 10, 1}, {0, 8, 10}, {8, 11, 10}},
        {{3, 9, 0}, {3, 11, 9}, {11, 10, 9}},
        {{9, 8, 10}, {10, 8, 11}},
        {{4, 7, 8}},
        {{4, 3, 0}, {7, 3, 4}},
        {{0, 1, 9}, {8, 4, 7}},
        {{4, 1, 9}, {4, 7, 1}, {7, 3, 1}},
        {{1, 2, 10}, {8, 4, 7}},
        {{3, 4, 7}, {3, 0, 4}, {1, 2, 10}},
        {{9, 2, 10}, {9, 0, 2}, {8, 4, 7}},
        {{2, 10, 9}, {2, 9, 7}, {2, 7, 3}, {7, 9, 4}},
        {{8, 4, 7}, {3, 11, 2}},
        {{11, 4, 7}, {11, 2, 4}, {2, 0, 4}},
        {{9, 0, 1}, {8, 4, 7}, {2, 3, 11}},
        {{4, 7, 11}, {9, 4, 11}, {9, 11, 2}, {9, 2, 1}},
        {{3, 10, 1}, {3, 11, 10}, {7, 8, 4}},
        {{1, 11, 10}, {1, 4, 11}, {1, 0, 4}, {7, 11, 4}},
        {{4, 7, 8}, {9, 0, 11}, {9, 11, 10}, {11, 0, 3}},
        {{4, 7, 11}, {4, 11, 9}, {9, 11, 10}},
        {{9, 5, 4}},
        {{9, 5, 4}, {0, 8, 3}},
        {{0, 5, 4}, {1, 5, 0}},
        {{8, 5, 4}, {8, 3, 5}, {3, 1, 5}},
        {{1, 2, 10}, {9, 5, 4}},
        {{3, 0, 8}, {1, 2, 10}, {4, 9, 5}},
        {{5, 2, 10}, {5, 4, 2}, {4, 0, 2}},
        {{2, 10, 5}, {3, 2, 5}, {3, 5, 4}, {3, 4, 8}},
        {{9, 5, 4}, {2, 3, 11}},
        {{0, 11, 2}, {0, 8, 11}, {4, 9, 5}},
        {{0, 5, 4}, {0, 1, 5}, {2, 3, 11}},
        {{2, 1, 5}, {2, 5, 8}, {2, 8, 11}, {4, 8, 5}},
        {{10, 3, 11}, {10, 1, 3}, {9, 5, 4}},
        {{4, 9, 5}, {0, 8, 1}, {8, 10, 1}, {8, 11, 10}},
        {{5, 4, 0}, {5, 0, 11}, {5, 11, 10}, {11, 0, 3}},
        {{5, 4, 8}, {5, 8, 10}, {10, 8, 11}},
        {{9, 7, 8}, {5, 7, 9}},
        {{9, 3, 0}, {9, 5, 3}, {5, 7, 3}},
        {{0, 7, 8}, {0, 1, 7}, {1, 5, 7}},
        {{1, 5, 3}, {3, 5, 7}},
        {{9, 7, 8}, {9, 5, 7}, {10, 1, 2}},
        {{10, 1, 2}, {9, 5, 0}, {5, 3, 0}, {5, 7, 3}},
        {{8, 0, 2}, {8, 2, 5}, {8, 5, 7}, {10, 5, 2}},
        {{2, 10, 5}, {2, 5, 3}, {3, 5, 7}},
        {{7, 9, 5}, {7, 8, 9}, {3, 11, 2}},
        {{9, 5, 7}, {9, 7, 2}, {9, 2, 0}, {2, 7, 11}},
        {{2, 3, 11}, {0, 1, 8}, {1, 7, 8}, {1, 5, 7}},
        {{11, 2, 1}, {11, 1, 7}, {7, 1, 5}},
        {{9, 5, 8}, {8, 5, 7}, {10, 1, 3}, {10, 3, 11}},
        {{5, 7, 0}, {5, 0, 9}, {7, 11, 0}, {1, 0, 10}, {11, 10, 0}},
        {{11, 10, 0}, {11, 0, 3}, {10, 5, 0}, {8, 0, 7}, {5, 7, 0}},
        {{11, 10, 5}, {7, 11, 5}},
        {{10, 6, 5}},
        {{0, 8, 3}, {5, 10, 6}},
        {{9, 0, 1}, {5, 10, 6}},
        {{1, 8, 3}, {1, 9, 8}, {5, 10, 6}},
        {{1, 6, 5}, {2, 6, 1}},
        {{1, 6, 5}, {1, 2, 6}, {3, 0, 8}},
        {{9, 6, 5}, {9, 0, 6}, {0, 2, 6}},
        {{5, 9, 8}, {5, 8, 2}, {5, 2, 6}, {3, 2, 8}},
        {{2, 3, 11}, {10, 6, 5}},
        {{11, 0, 8}, {11, 2, 0}, {10, 6, 5}},
        {{0, 1, 9}, {2, 3, 11}, {5, 10, 6}},
        {{5, 10, 6}, {1, 9, 2}, {9, 11, 2}, {9, 8, 11}},
        {{6, 3, 11}, {6, 5, 3}, {5, 1, 3}},
        {{0, 8, 11}, {0, 11, 5}, {0, 5, 1}, {5, 11, 6}},
        {{3, 11, 6}, {0, 3, 6}, {0, 6, 5}, {0, 5, 9}},
        {{6, 5, 9}, {6, 9, 11}, {11, 9, 8}},
        {{5, 10, 6}, {4, 7, 8}},
        {{4, 3, 0}, {4, 7, 3}, {6, 5, 10}},
        {{1, 9, 0}, {5, 10, 6}, {8, 4, 7}},
        {{10, 6, 5}, {1, 9, 7}, {1, 7, 3}, {7, 9, 4}},
        {{6, 1, 2}, {6, 5, 1}, {4, 7, 8}},
        {{1, 2, 5}, {5, 2, 6}, {3, 0, 4}, {3, 4, 7}},
        {{8, 4, 7}, {9, 0, 5}, {0, 6, 5}, {0, 2, 6}},
        {{7, 3, 9}, {7, 9, 4}, {3, 2, 9}, {5, 9, 6}, {2, 6, 9}},
        {{3, 11, 2}, {7, 8, 4}, {10, 6, 5}},
        {{5, 10, 6}, {4, 7, 2}, {4, 2, 0}, {2, 7, 11}},
        {{0, 1, 9}, {4, 7, 8}, {2, 3, 11}, {5, 10, 6}},
        {{9, 2, 1}, {9, 11, 2}, {9, 4, 11}, {7, 11, 4}, {5, 10, 6}},
        {{8, 4, 7}, {3, 11, 5}, {3, 5, 1}, {5, 11, 6}},
        {{5, 1, 11}, {5, 11, 6}, {1, 0, 11}, {7, 11, 4}, {0, 4, 11}},
        {{0, 5, 9}, {0, 6, 5}, {0, 3, 6}, {11, 6, 3}, {8, 4, 7}},
        {{6, 5, 9}, {6, 9, 11}, {4, 7, 9}, {7, 11, 9}},
        {{10, 4, 9}, {6, 4, 10}},
        {{4, 10, 6}, {4, 9, 10}, {0, 8, 3}},
        {{10, 0, 1}, {10, 6, 0}, {6, 4, 0}},
        {{8, 3, 1}, {8, 1, 6}, {8, 6, 4}, {6, 1, 10}},
        {{1, 4, 9}, {1, 2, 4}, {2, 6, 4}},
        {{3, 0, 8}, {1, 2, 9}, {2, 4, 9}, {2, 6, 4}},
        {{0, 2, 4}, {4, 2, 6}},
        {{8, 3, 2}, {8, 2, 4}, {4, 2, 6}},
        {{10, 4, 9}, {10, 6, 4}, {11, 2, 3}},
        {{0, 8, 2}, {2, 8, 11}, {4, 9, 10}, {4, 10, 6}},
        {{3, 11, 2}, {0, 1, 6}, {0, 6, 4}, {6, 1, 10}},
        {{6, 4, 1}, {6, 1, 10}, {4, 8, 1}, {2, 1, 11}, {8, 11, 1}},
        {{9, 6, 4}, {9, 3, 6}, {9, 1, 3}, {11, 6, 3}},
        {{8, 11, 1}, {8, 1, 0}, {11, 6, 1}, {9, 1, 4}, {6, 4, 1}},
        {{3, 11, 6}, {3, 6, 0}, {0, 6, 4}},
        {{6, 4, 8}, {11, 6, 8}},
        {{7, 10, 6}, {7, 8, 10}, {8, 9, 10}},
        {{0, 7, 3}, {0, 10, 7}, {0, 9, 10}, {6, 7, 10}},
        {{10, 6, 7}, {1, 10, 7}, {1, 7, 8}, {1, 8, 0}},
        {{10, 6, 7}, {10, 7, 1}, {1, 7, 3}},
        {{1, 2, 6}, {1, 6, 8}, {1, 8, 9}, {8, 6, 7}},
        {{2, 6, 9}, {2, 9, 1}, {6, 7, 9}, {0, 9, 3}, {7, 3, 9}},
        {{7, 8, 0}, {7, 0, 6}, {6, 0, 2}},
        {{7, 3, 2}, {6, 7, 2}},
        {{2, 3, 11}, {10, 6, 8}, {10, 8, 9}, {8, 6, 7}},
        {{2, 0, 7}, {2, 7, 11}, {0, 9, 7}, {6, 7, 10}, {9, 10, 7}},
        {{1, 8, 0}, {1, 7, 8}, {1, 10, 7}, {6, 7, 10}, {2, 3, 11}},
        {{11, 2, 1}, {11, 1, 7}, {10, 6, 1}, {6, 7, 1}},
        {{8, 9, 6}, {8, 6, 7}, {9, 1, 6}, {11, 6, 3}, {1, 3, 6}},
        {{0, 9, 1}, {11, 6, 7}},
        {{7, 8, 0}, {7, 0, 6}, {3, 11, 0}, {11, 6, 0}},
        {{7, 11, 6}}
    };
    //</editor-fold>

    private RawSparseZYX rawTZYXWithPadding0;
    private double xSp, ySp, zSp;
}
