/*
 * USE OF THE LIFEx SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS AND CONDITIONS. 
 * IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT DOWNLOAD THE SOFTWARE. 
 * NON-PROFESSIONAL USERS ARE NOTIFIED THAT THE SOFTWARE AND THE RISKS INHERENT TO ITS USE,
 * MODIFICATION AND/OR DEVELOPMENT AND REPRODUCTION ARE NOT COVERED BY ANY WARRANTY ON THE PART OF THE CEA. 
 * IT IS HIGHLY RECOMMENDED THAT YOU ONLY LOAD AND/OR USE THE SOFTWARE IF YOU HAVE SPECIALIZED IN-DEPTH INFORMATION TECHNOLOGY KNOW-HOW.
 * THE COMMISSARIAT A L’ENERGIE ATOMIQUE ET AUX ENERGIES ALTERNATIVES (FRENCH ATOMIC ENERGY AGENCY), 
 * a public scientific, technical and industrial establishment, 
 * having its registered office at 25 rue Leblanc – bâtiment le Ponant D – 75015 Paris cedex, 
 * hereinafter referred to as "CEA", hereby licenses the right to use the LIFEx software as per the terms and conditions set out below.  
 */
package bardp.protocol.protocolTexture.textureGLZLM;

import java.util.HashMap;

/**
 * <p>
 * Title: Local Image Feature Extraction (LIFEx)</p>
 * <p>
 * Description: Management of Sparse Matrix for GLZLM</p>
 * <p>
 * Copyright: Copyright (c) 2016</p>
 * <p>
 * Company: Inserm/CNRS/CEA</p>
 * <p>
 * @author Christophe Nioche, Inserm at lifexsoft.org
 * @version 6.0
 */
public class TextureGLZLMSparse {

    public final HashMap hashtableRow = new HashMap();

    public TextureGLZLMSparse() {
    }
        
    /**
     * set value at location iRow, iColumn indices.
     * <br>if index doesn't exist at this location then it is create and value is setting
     * <br>if index exist the value is replaced
     * <br>if the value is zero, then index (and his value) is removed
     *
     * @param iRow index of row
     * @param iColumn index of column
     * @param value value to set
     */
    public void set(
            final int iRow, 
            final int iColumn, 
            final short value
    ) {
        HashMap _hashtableRow = (HashMap) hashtableRow.get(iRow);

        if (_hashtableRow != null) {
            if (value != 0) {
                _hashtableRow.put(iColumn, value); // add or replace
            } else if (_hashtableRow.containsKey(iColumn)) {
                _hashtableRow.remove(iColumn);    // remove if exist
            }

        } else if (value != 0) {
            _hashtableRow = new HashMap();
            _hashtableRow.put(iColumn, value);

            hashtableRow.put(iRow, _hashtableRow);
        }
    }
  
    
     /**
     * Increase of 1 a value at iRow, iColumn location.
     * <br>if index doesn't exist then it's created with value 1
     * <br>if index exist his value is increased of 1
     *
     * @param iRow to define
     * @param iColumn to define
     */
    public void setInc(
            final int iRow, 
            final int iColumn
    ) {
        HashMap _hashtableRow = (HashMap) hashtableRow.get(iRow);

        if (_hashtableRow != null) {
            if (_hashtableRow.get(iColumn)!=null) {
                short value = (short)_hashtableRow.get(iColumn);    
                _hashtableRow.put(iColumn, ++value);
            } else {
                _hashtableRow.put(iColumn, (short)1);
            }

        } else {
            _hashtableRow = new HashMap();
            _hashtableRow.put(iColumn, (short)1);

            hashtableRow.put(iRow, _hashtableRow);
        }
    }
    
    public short get(
            final int iRow,
            final int iColumn
    ) {
        try {
            HashMap _hashtableRow = (HashMap) hashtableRow.get(iRow);
            return (short) _hashtableRow.get(iColumn);
        } catch (NullPointerException ex) {
            return (short) 0;
        }
    }
    
    public void del(
            final int iRow, 
            final int iColumn
    ) {
        HashMap _hashtableRow = (HashMap) hashtableRow.get(iRow);
        if (_hashtableRow != null) {
             if (_hashtableRow.containsKey(iColumn)) {
            _hashtableRow.remove(iColumn);
             }
        }
    }
}
